package main

import (
	"encoding/json"
	"fmt"
	"math/rand"
	"os"
	"path"
	"strings"
	"time"

	"github.com/gdamore/tcell/v2"
	"github.com/rivo/tview"
)

func getTitle() string {
	titles := []string{
		"yo bum rush the show",
		"can i kick it?",
		"super nintendo sega genesis",
		"birthdays was the worst days",
		"where were you when we were getting high?",
		"it's real time, real time, time to get real",
	}
	return titles[rand.Intn(len(titles))]
}

const (
	signupDir   = "/town/signups"
	acceptedDir = "/town/signups/accepted"
	rejectedDir = "/town/signups/rejected"
)

type townSignup struct {
	When    time.Time
	Answers map[string]string
}

func (a townSignup) Render() string {
	out := fmt.Sprintf("[-:-:b]submitted:[-:-:-] %s\n", a.When.Format("2006-01-02 15:04"))

	for k, v := range a.Answers {
		out += fmt.Sprintf("[-:-:b]%s[-:-:-]\n", k)
		out += strings.TrimSpace(v)
		out += "\n\n"
	}

	return out
}

func getSignups() ([]townSignup, error) {
	entries, err := os.ReadDir(signupDir)
	if err != nil {
		return nil, fmt.Errorf("failed to read '%s': %w", signupDir, err)
	}
	out := []townSignup{}
	for _, entry := range entries {
		if !strings.HasSuffix(entry.Name(), "json") {
			continue
		}

		abs := path.Join(signupDir, entry.Name())
		data, err := os.ReadFile(abs)
		if err != nil {
			return nil, fmt.Errorf("could not read signup file '%s': %w", abs, err)
		}

		fmt.Println(string(data))

		var signup townSignup
		err = json.Unmarshal(data, &signup)
		if err != nil {
			return nil, fmt.Errorf("could not unmarshal signup file '%s': %w", abs, err)
		}

		out = append(out, signup)
	}

	return out, nil
}

func _main() error {
	rand.Seed(time.Now().Unix())

	signups, err := getSignups()
	if err != nil {
		return fmt.Errorf("could not get signups: %w", err)
	}

	signupIx := 0

	pages := tview.NewPages()
	mainFlex := tview.NewFlex()

	title := tview.NewTextView()
	title.SetText(getTitle())
	title.SetTextAlign(tview.AlignCenter)
	title.SetTextColor(tcell.ColorPink)
	title.SetBackgroundColor(tcell.ColorBlack)

	appView := tview.NewTextView()
	appView.SetDynamicColors(true)
	if len(signups) == 0 {
		appView.SetText("no signups found.")
	} else {
		appView.SetText(signups[signupIx].Render())
	}

	legend := tview.NewTextView()
	legend.SetText("TODO actions legend")

	mainFlex.SetDirection(tview.FlexRow)
	mainFlex.AddItem(title, 1, -1, false)
	mainFlex.AddItem(appView, 0, 1, true)
	mainFlex.AddItem(legend, 1, -1, false)

	pages.AddPage("main", mainFlex, true, true)
	// TODO page for textarea to notate

	app := tview.NewApplication()
	app.SetRoot(pages, true)

	app.SetInputCapture(func(event *tcell.EventKey) *tcell.EventKey {
		switch event.Rune() {
		case 's':
			signupIx++
			if signupIx == len(signups) {
				signupIx = 0
			}
			appView.SetText(signups[signupIx].Render())
		case 'r':
			signupIx = rand.Intn(len(signups))
			appView.SetText(signups[signupIx].Render())
		case 'A':
			// TODO approve
		case 'R':
			// TODO reject
		case 'n':
			// TODO notate
		case 'Q':
			app.Stop()
		}

		return event
	})

	return app.Run()
}

func main() {
	err := _main()
	if err != nil {
		fmt.Fprintln(os.Stderr, err)
		os.Exit(1)
	}
}
